/*--------------------------------------------------------------------*
 *
 * Developed by;
 *	Neal Horman - http://www.wanlink.com
 *	Copyright (c) 2002 Neal Horman. All Rights Reserved
 *
 *	Redistribution and use in source and binary forms, with or without
 *	modification, are permitted provided that the following conditions
 *	are met:
 *	1. Redistributions of source code must retain the above copyright
 *	   notice, this list of conditions and the following disclaimer.
 *	2. Redistributions in binary form must reproduce the above copyright
 *	   notice, this list of conditions and the following disclaimer in the
 *	   documentation and/or other materials provided with the distribution.
 *	3. All advertising materials mentioning features or use of this software
 *	   must display the following acknowledgement:
 *	This product includes software developed by Neal Horman.
 *	4. Neither the name Neal Horman nor the names of any contributors
 *	   may be used to endorse or promote products derived from this software
 *	   without specific prior written permission.
 *	
 *	THIS SOFTWARE IS PROVIDED BY NEAL HORMAN AND ANY CONTRIBUTORS ``AS IS'' AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *	ARE DISCLAIMED.  IN NO EVENT SHALL NEAL HORMAN OR ANY CONTRIBUTORS BE LIABLE
 *	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *	OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *	LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *	OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *	SUCH DAMAGE.
 *
 *	CVSID:  $Id: ipfw_direct.c,v 1.1 2004/04/10 01:35:43 neal Exp $
 *
 * DESCRIPTION:
 *	application:	ipfwmtad
 *	module:		ipfw_direct.c
 *--------------------------------------------------------------------*/

#include <sys/param.h>
#include <sys/mbuf.h>
#include <sys/socket.h>
#include <sys/sockio.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <net/if.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netinet/ip_fw.h>
#include <net/route.h> /* def. of struct route */
#include <netinet/ip_dummynet.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

extern int debugmode;

int ipfw_sd = -1;

int ipfw_add(int rulenum, long ip, short port)
{	int rc = -1;

	if(ipfw_sd != -1)
	{	struct ip_fw	rule;
		u_short		*sp;
		int		i;
	
		memset(&rule, 0, sizeof rule);

		rule.fw_number	= rulenum;
		rule.fw_flg	|= IP_FW_F_DENY;
		rule.fw_prot	= IPPROTO_TCP;

		/* from */
		rule.fw_src.s_addr	= htonl(ip);
		rule.fw_smsk.s_addr	= htonl(~0);

		/* to */
		rule.fw_dst.s_addr	= 0;
		rule.fw_dmsk.s_addr	= 0;

		*(rule.fw_uar.fw_pts+IP_FW_GETNSRCP(&rule)) = port;
		IP_FW_SETNDSTP(&rule, 1);

		/* No direction specified -> do both directions */
		rule.fw_flg |= (IP_FW_F_OUT|IP_FW_F_IN);

		i = sizeof(rule);
		rc = getsockopt(ipfw_sd, IPPROTO_IP, IP_FW_ADD, &rule, &i);

		if(debugmode > 1)
			printf("ipfw_add: %u.%u.%u.%u port %u\n",(ip&0xff000000)>>24,(ip&0x00ff0000)>>16,(ip&0x0000ff00)>>8,ip&0xff,port);
	}

	return(rc == 0);
}

int ipfw_del(int rulenum)
{	int	rc = -1;

	if(ipfw_sd != -1)
	{	struct ip_fw	rule;

		memset(&rule, 0, sizeof rule);
		rule.fw_number = rulenum;
		rc = setsockopt(ipfw_sd, IPPROTO_IP, IP_FW_DEL, &rule, sizeof(rule));
	}

	return(rc == 0);
}

void ipfw_startup()
{	
	if(ipfw_sd == -1)
		ipfw_sd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW);
}

void ipfw_shutdown()
{
	if(ipfw_sd != -1)
	{
		close(ipfw_sd);
		ipfw_sd = -1;
	}
}

/*--------------------------------------------------------------------*
 * $Log: ipfw_direct.c,v $
 * Revision 1.1  2004/04/10 01:35:43  neal
 * neal - 040409 - talk directly to the ipfw api that ipfw cli util uses instead of execing a bunch
 *
 *--------------------------------------------------------------------*/
